//============================================================================
// Bare-Bones Tablet Interface for C++
//
//  This header was designed to be platform-neutral for portability, 
//  but currently Win32 is the only implemented back end.
//
//  Relies on Wintab32.dll for windows operation.
//  If you have a tablet, then you should have a copy of Wintab32.dll that
//  was installed by the tablet's driver installer.
//  
//  Also necessary (for development on Windows) will be the WINTAB SDK.
//  It can be downloaded for free from http://www.lcs-telegraphics.com/
//  It contains the necessary headers: wintab.h, pktdef.h ...
//
//  Author:        William Baxter (baxter@cs.unc.edu)
//  Last Modified: Aug 2002
//============================================================================
//  Copyright 2002 
//        William Baxter and
//        The University of North Carolina at Chapel Hill
// 
//  Permission to use, copy, modify, distribute and sell this software
//  and its documentation for any purpose is hereby granted without
//  fee, provided that the above copyright notice appear in all copies
//  and that both that copyright notice and this permission notice
//  appear in supporting documentation.  Binaries may be compiled with
//  this software without any royalties or restrictions.
// 
//  The University of North Carolina at Chapel Hill makes no
//  representations about the suitability of this software for any
//  purpose. It is provided "as is" without express or implied
//  warranty.
//============================================================================


#ifndef BB_TABLET_HEADER
#define BB_TABLET_HEADER

#if defined(_WIN32) && !defined(BBTAB_NOAUTOLINK)
#ifdef _DLL
 // Decide what version of bbtablet.lib to link with and auto-link it.
 #ifdef _DEBUG
 #pragma comment (lib, "bbtabletds.lib") // for /MDd apps
 #else
 #pragma comment (lib, "bbtablets.lib") // for /MD apps
 #endif
#else 
 #ifdef _DEBUG
 #pragma comment (lib, "bbtabletd.lib") // for /MTd, /MLd apps
 #else 
 #pragma comment (lib, "bbtablet.lib") // for /ML, /MT apps
 #endif
#endif
#endif


/**
 * @class bbTabletEvent
 */
class bbTabletEvent
{
public:
  float x, y, z;
  float angle;
  float axis[3];
  float pressure;
  float tpressure;
  unsigned int id;      // unique physical id if supported
  unsigned int buttons; // bitmask of pressed buttons
  int type; // pass to getCursor* methods
};


class _bbTabletDeviceInternal;

/**
 * @class bbTabletDevice
 *
 *   This class is a singleton. 
 *   Wintab may be able to support multiple tablets,
 *   but I don't think Wacom's drivers can support it.
 */
class bbTabletDevice 
{
private:
  // bbTabletDevice is a singleton, so it can't be user-instantiated or copied
  // Use getInstance() instead.
  bbTabletDevice(); 
  bbTabletDevice(const &bbTabletDevice);

public:
  enum { SYSTEM_POINTER, SEPARATE_POINTER };

  static bbTabletDevice &getInstance();

  /** 
   * initTablet
   * @param system_window  should be an HWND on Windows, e.g.
   * @param mode 
   *      SYSTEM_POINTER if this tablet is to drive  the system cursor
   *      SEPARATE_POINTER if this tablet doesn't affect the system cursor
   * @return true if successfully initialized
   */
  bool initTablet( void *system_window, int mode = SYSTEM_POINTER );

  /**
   * setPointerMode
   * @param mode
   *      SYSTEM_POINTER if this tablet is to drive  the system cursor
   *      SEPARATE_POINTER if this tablet doesn't affect the system cursor
   * @return true if pointer mode successfully set
   */
  bool setPointerMode( int mode );
  /**
   * pointerMode
   * Returns the current pointer mode, either SYSTEM_PONITER or SEPARATE_POINTER
   */
  int pointerMode( ) const;
  /**
   * isValid
   *    Returns true if the tablet has been successfully initialized
   */
  bool isValid() const;
  
  bool getNextEvent(bbTabletEvent &evt);
  bool waitForNextEvent(bbTabletEvent &evt, unsigned int msec = 0);

  const char *getCursorName(int csrtype);
  int getCursorNumButtons(int csrtype);
  const char *getCursorButtonName(int csrtype, int buttonnum);

private:
  _bbTabletDeviceInternal *m_data;
};



#endif /* BB_TABLET_HEADER */
